import inspect
from typing import TypeVar, Type, Any, Generic, GenericMeta, cast, Generator, Tuple

from amino import Map, Lists, List, Nil, _, Either, Right, Maybe, Just
from amino.util.string import ToStr
from amino.func import Val
from amino.do import tdo
from amino.lazy import lazy
from amino.json.decoder import Decoder
from amino.json.data import JsonError, JsonObject, JsonScalar
from amino.json.encoder import Encoder, encode_json
from amino.algebra import AlgebraMeta, Algebra

A = TypeVar('A')


class KeepField:
    pass


def qualified_type(tpe: Type) -> str:
    return tpe.__name__ if tpe.__module__ == 'builtins' else f'{tpe.__module__}.{tpe.__name__}'


class Field(ToStr):

    def __init__(self, name: str, tpe: Type) -> None:
        self.name = name
        self.tpe = tpe

    @property
    def qualified_type(self) -> None:
        return qualified_type(self.tpe)

    def _arg_desc(self) -> List[str]:
        return List(self.name, self.qualified_type)

    @property
    def param_str(self) -> str:
        return f'''{self.name}: '{self.qualified_type}'=Dat.Keep'''


Sub = TypeVar('Sub', bound='Dat')


class FieldMutator(Generic[Sub]):

    def __init__(self, name: str, target: 'Dat[Sub]') -> None:
        self.name = name
        self.target = target


class FieldSetter(Generic[Sub], FieldMutator[Sub]):

    def __call__(self, value) -> Sub:
        return self.target.copy(**{self.name: value})


class FieldModifier(Generic[Sub], FieldMutator[Sub]):

    def __call__(self, f) -> Sub:
        return self.target.copy(**{self.name: f(getattr(self.target, self.name))})


class FieldAppender(Generic[Sub], FieldMutator[Sub]):

    def __call__(self, value) -> Sub:
        return self.target.mod(self.name)(_ + value)


class FieldAppender1(Generic[Sub], FieldMutator[Sub]):

    def __call__(self, value) -> Sub:
        return self.target.mod(self.name)(_ + List(value))


class FieldProxy(Generic[Sub], FieldMutator[Sub]):

    def __init__(self, target: 'Dat', tpe: type) -> None:
        self.target = target
        self.tpe = tpe

    def __getattr__(self, name):
        return self(name)

    def __call__(self, name):
        return self.tpe(name, self.target)


def init_fields(spec: inspect.FullArgSpec) -> List[Field]:
    args = Lists.wrap(spec.args).tail | Nil
    types = Map(spec.annotations)
    def field(name: str) -> Field:
        tpe = types.lift(name) | Val(Any)
        return Field(name, tpe)
    return args / field


class DatMeta(GenericMeta):

    def __new__(cls: type, name: str, bases: tuple, namespace: dict, **kw) -> type:
        fs = Map(namespace).lift('__init__') / inspect.getfullargspec / init_fields | Nil
        inst = super().__new__(cls, name, bases, namespace, **kw)
        inst._dat__fields_value = fs
        return inst

    @property
    def _dat__fields(self) -> List[Field]:
        return self._dat__fields_value

    @property
    def _field_count_min(self) -> int:
        return len(self._dat__fields)

    @property
    def _field_count_max(self) -> Maybe[int]:
        return Just(len(self._dat__fields))


class Dat(Generic[Sub], ToStr, metaclass=DatMeta):
    Keep = KeepField()

    @property
    def _dat__fields(self) -> List[Field]:
        return type(self)._dat__fields

    @lazy
    def _dat__names(self) -> List[str]:
        return self._dat__fields.map(_.name)

    @lazy
    def _dat__values(self) -> List[Any]:
        return (
            self._dat__fields
            .traverse(lambda a: Maybe.getattr(self, a.name), Maybe)
            .get_or_fail(lambda: f'corrupt `Dat`: {type(self)}')
        )

    @lazy
    def _dat__items(self) -> List[Tuple[str, Any]]:
        return self._dat__names.zip(self._dat__values)

    @property
    def to_map(self) -> Map[str, Any]:
        return Map(self._dat__items)

    def copy(self, **kw: Any) -> Sub:
        updates = Map(kw)
        def update(f: Field) -> Any:
            return updates.lift(f.name) | getattr(self, f.name)
        updated = self._dat__fields / update  # type: ignore
        return cast(Dat, type(self)(*updated))

    @property
    def append(self) -> FieldProxy:
        return FieldProxy(self, FieldAppender)

    @property
    def append1(self) -> FieldProxy:
        return FieldProxy(self, FieldAppender1)

    @property
    def mod(self) -> FieldProxy:
        return FieldProxy(self, FieldModifier)

    @property
    def set(self) -> FieldProxy:
        return FieldProxy(self, FieldSetter)

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, type(self)) and self._dat__values == other._dat__values

    def _lens_setattr(self, name, value):
        return self.set(name)(value)

    def _arg_desc(self) -> List[str]:
        return self._dat__values / str


class DatDecoder(Decoder, tpe=Dat):

    def decode(self, tpe: Type[Sub], data: JsonObject) -> Either[JsonError, Sub]:
        @tdo(Either[JsonError, A])
        def decode_field(field: Field) -> Generator:
            value = yield data.field(field.name).to_either(f'missing field {field.name} in json while decoding {tpe}')
            dec = yield Decoder.e(field.tpe)
            yield dec.decode(field.tpe, value)
        return tpe._dat__fields.traverse(decode_field, Either).map(lambda a: tpe(*a))


class DatEncoder(Encoder, tpe=Dat):

    @tdo(Either[JsonError, Map])
    def encode(self, a: Sub) -> Generator:
        jsons = yield a._dat__values.traverse(encode_json, Either)
        yield Right(JsonObject(Map(a._dat__names.zip(jsons)).cat(('__type__', JsonScalar(qualified_type(type(a)))))))


class ADTMeta(DatMeta, AlgebraMeta):
    pass


class ADT(Generic[Sub], Dat[Sub], Algebra, metaclass=ADTMeta):
    pass

__all__ = ('Dat', 'ADT')



import abc
from typing import TypeVar, Generator, Any, Type, Generic

from amino.tc.base import TypeClass
from amino import Either, Map, Right, Lists
from amino.do import tdo
from amino.json.data import JsonError, JsonObject, JsonArray, JsonScalar, Json
from amino.json.parse import parse_json
from amino.dispatch import dispatch_alg

A = TypeVar('A')


class Decoder(Generic[A], TypeClass):

    @abc.abstractmethod
    def decode(self, tpe: Type[A], data: Map[str, Any]) -> Either[JsonError, A]:
        ...


@tdo(Either[str, A])
def decode_json_object(data: dict) -> Generator:
    m = Map(data)
    tpe_s = yield m.get('__type__').to_either(f'no `__type__` attr in json object {m}')
    tpe = yield Either.import_path(tpe_s)
    dec = yield Decoder.e(tpe)
    yield dec.decode(tpe, m)


class Decode:

    @tdo(Either[str, A])
    def decode_json_object(self, json: JsonObject) -> Either[str, A]:
        tpe = yield json.tpe
        dec = yield Decoder.e(tpe)
        yield dec.decode(tpe, json)

    def decode_json_array(self, json: JsonArray) -> Either[str, A]:
        return Lists.wrap(json.data).traverse(decode, Either)

    def decode_json_scalar(self, json: JsonScalar) -> Either[str, A]:
        return Right(json.data)


decode = dispatch_alg(Decode(), Json, 'decode_')


@tdo(Either[JsonError, A])
def decode_json(data: str) -> Generator:
    json = yield parse_json(data)
    yield decode(json)

__all__ = ('Decoder', 'decode_json_object', 'decode_json')



from typing import Type, TypeVar, Collection
from numbers import Number

from amino.json.decoder import Decoder, decode
from amino import Maybe, Either, List, Lists, Left
from amino.json.data import JsonError, Json

A = TypeVar('A')


class StringDecoder(Decoder, tpe=str):

    def decode(self, tpe: Type[str], data: Json) -> Either[JsonError, str]:
        return data.scalar.e(f'invalid type for `str`: {data}', data.data)


class NumberDecoder(Decoder, sub=Number):

    def decode(self, tpe: Type[int], data: Json) -> Either[JsonError, int]:
        return data.scalar.e(f'invalid type for `int`: {data}', data.data)


class ListDecoder(Decoder, sub=Collection):

    def decode(self, tpe: Type[Collection], data: Json) -> Either[JsonError, List[A]]:
        def dec() -> Either[JsonError, List[A]]:
            return Lists.wrap(data.data).traverse(decode, Either)
        return data.array.c(dec, lambda: Left(f'invalid type for `List`: {data}'))


class MaybeDecoder(Decoder, tpe=Maybe):

    def decode(self, tpe: Type[Maybe], data: Json) -> Either[JsonError, Maybe]:
        return data.scalar.e(f'invalid type for `Maybe`: {data}', Maybe.check(data.data))

__all__ = ('MaybeDecoder',)



import abc
from typing import TypeVar, Union, Generic, Generator
from numbers import Number

from amino import Map, List, _, Either, Right, Left, Boolean
from amino.util.string import ToStr
from amino.algebra import Algebra
from amino.do import tdo

A = TypeVar('A')


class JsonError(ToStr):

    def __init__(self, data: str, error: Union[str, Exception]) -> None:
        self.data = data
        self.error = error

    def _arg_desc(self) -> List[str]:
        return List(self.data, str(self.error))


class Json(Generic[A], Algebra, base=True):

    def __init__(self, data: A) -> None:
        self.data = data

    @abc.abstractproperty
    def native(self) -> Union[dict, list, str, Number, None]:
        ...

    def _arg_desc(self) -> List[str]:
        return List(str(self.data))

    @property
    def scalar(self) -> Boolean:
        return Boolean.isinstance(self, JsonScalar)

    @property
    def array(self) -> Boolean:
        return Boolean.isinstance(self, JsonArray)


class JsonObject(Json[Map[str, Json]]):

    @property
    def native(self) -> Union[dict, list, str, Number, None]:
        return self.data.valmap(_.native)

    @property
    @tdo(Either[str, type])
    def tpe(self) -> Generator:
        jtpe = yield self.data.lift('__type__').to_either(f'no `__type__` field in json object {self.data}')
        tpe_s = yield Right(jtpe.data) if isinstance(jtpe, JsonScalar) else Left('invalid type for `__type__`: {jtpe}')
        yield Either.import_path(tpe_s)

    def field(self, key: str) -> Either[str, Json]:
        return self.data.lift(key).to_either(f'no field `key` in `self`')


class JsonArray(Json[List[Json]]):

    @property
    def native(self) -> Union[dict, list, str, Number, None]:
        return self.data.map(_.native)


class JsonScalar(Json[Union[str, Number, None]]):

    @property
    def native(self) -> Union[dict, list, str, Number, None]:
        return self.data

__all__ = ('JsonError', 'Json', 'JsonObject', 'JsonArray', 'JsonScalar')



from typing import Union, Collection, TypeVar
from numbers import Number

from amino import Either, List, L, _, Right, Lists, Maybe
from amino.json.encoder import Encoder, encode_json
from amino.json.data import JsonError, Json, JsonArray, JsonScalar

A = TypeVar('A')


class ScalarEncoder(Encoder[Union[Number, str, None]], pred=L(issubclass)(_, (Number, str, type(None)))):

    def encode(self, a: Union[Number, str, None]) -> Either[JsonError, Json]:
        return Right(JsonScalar(a))


class ListEncoder(Encoder[List], pred=L(issubclass)(_, Collection)):

    def encode(self, a: Collection) -> Either[JsonError, Json]:
        return Lists.wrap(a).traverse(encode_json, Either) / JsonArray


class MaybeEncoder(Encoder[Maybe], tpe=Maybe):

    def encode(self, a: Maybe[A]) -> Either[JsonError, Json]:
        return Right(JsonScalar(a | None))

__all__ = ('ListEncoder', 'ScalarEncoder')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# __coconut_hash__ = 0x404ec769

# Compiled with Coconut version 1.3.0 [Dead Parrot]

# Coconut Header: -------------------------------------------------------------

from __future__ import generator_stop
import sys as _coconut_sys, os.path as _coconut_os_path
_coconut_file_path = _coconut_os_path.dirname(_coconut_os_path.abspath(__file__))
_coconut_sys.path.insert(0, _coconut_file_path)
from __coconut__ import _coconut, _coconut_NamedTuple, _coconut_MatchError, _coconut_tail_call, _coconut_tco, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_pipe, _coconut_star_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial
from __coconut__ import *
_coconut_sys.path.remove(_coconut_file_path)

# Compiled Coconut: -----------------------------------------------------------

import abc  # line 1
import importlib  # line 2
from typing import TypeVar  # line 3
from typing import Generic  # line 3
from typing import Union  # line 3
from typing import Any  # line 3
from typing import cast  # line 3
from typing import Iterator  # line 3
from typing import Type  # line 3
from types import ModuleType  # line 4
from pathlib import Path  # line 5

import amino  # line 7
from amino import boolean  # line 8
from amino.func import I  # line 9
from amino.tc.base import F  # line 10
from amino.util.mod import unsafe_import_name  # line 11
from amino.tc.monoid import Monoid  # line 12
from amino.util.string import ToStr  # line 13
from amino.do import do  # line 14
from amino.util.exception import format_exception  # line 15

A = TypeVar('A')  # line 17
B = TypeVar('B')  # line 18
C = TypeVar('C')  # line 19
D = TypeVar('D')  # line 20
E = TypeVar('E', bound=Exception)  # line 21


class ImportFailure(ToStr):  # line 24

    @abc.abstractproperty  # line 26
    def expand(self) -> 'amino.List[str]':  # line 27
        ...  # line 28


class ImportException(ImportFailure):  # line 31

    def __init__(self, desc: 'str', exc: 'Exception') -> 'None':  # line 33
        self.desc = desc  # line 34
        self.exc = exc  # line 35

    @_coconut_tco  # line 37
    def _arg_desc(self) -> 'amino.List[str]':  # line 37
        return _coconut_tail_call(amino.List, self.desc, str(self.exc))  # line 38

    @property  # line 40
    @_coconut_tco  # line 40
    def expand(self) -> 'amino.List[str]':  # line 41
        return _coconut_tail_call(format_exception(self.exc).cons, self.desc)  # line 42


class InvalidLocator(ImportFailure):  # line 45

    def __init__(self, msg: 'str') -> 'None':  # line 47
        self.msg = msg  # line 48

    @_coconut_tco  # line 50
    def _arg_desc(self) -> 'amino.List[str]':  # line 50
        return _coconut_tail_call(amino.List, self.msg)  # line 51

    @property  # line 53
    @_coconut_tco  # line 53
    def expand(self) -> 'amino.List[str]':  # line 54
        return _coconut_tail_call(amino.List, self.msg)  # line 55


class Either(Generic[A, B], F[B], implicits=True):  # line 58

    @staticmethod  # line 60
    @_coconut_tco  # line 60
    def import_name(mod: 'str', name: 'str') -> 'Either[ImportFailure, B]':  # line 61
        try:  # line 62
            value = unsafe_import_name(mod, name)  # line 63
        except Exception as e:  # line 64
            return _coconut_tail_call(Left, ImportException(f'{mod}.{name}', e))  # line 65
        else:  # line 66
            return Left(InvalidLocator(f'{mod} has no attribute {name}')) if value is None else Right(value)  # line 67

    @staticmethod  # line 69
    def import_path(path: 'str') -> 'Either[ImportFailure, B]':  # line 70
        from amino.list import List  # line 71
        return (List.wrap(path.rsplit('.', 1)).lift_all(0, 1).to_either(InvalidLocator(f'invalid module path: {path}')).flat_map2(lambda a, b: Either.import_name(a, b).lmap(lambda c: ImportException(path, c))))  # line 72

    @staticmethod  # line 79
    @_coconut_tco  # line 79
    def import_module(modname: 'str') -> 'Either[ImportFailure, ModuleType]':  # line 80
        try:  # line 81
            mod = importlib.import_module(modname)  # line 82
        except Exception as e:  # line 83
            return _coconut_tail_call(Left, ImportException(modname, e))  # line 84
        else:  # line 85
            return _coconut_tail_call(Right, mod)  # line 86

    @staticmethod  # line 88
    @_coconut_tco  # line 88
    def import_file(path: 'Path') -> 'Either[ImportFailure, ModuleType]':  # line 89
        from amino.maybe import Maybe  # line 90
        def step2(spec: 'importlib._bootstrap.ModuleSpec') -> 'ModuleType':  # line 91
            module = importlib.util.module_from_spec(spec)  # line 92
            spec.loader.exec_module(module)  # line 93
            return module  # line 94
        try:  # line 95
            module = Maybe.check(importlib.util.spec_from_file_location('temp', str(path))) / step2  # line 96
        except Exception as e:  # line 97
            return _coconut_tail_call(Left, ImportException(str(path), e))  # line 98
        else:  # line 99
            return _coconut_tail_call(module.to_either, InvalidLocator(f'failed to import `{path}`'))  # line 100

    @staticmethod  # line 102
    @do  # line 102
    def import_from_file(path: 'Path', name: 'str') -> 'Either[ImportFailure, B]':  # line 104
        module = yield Either.import_file(path)  # line 105
        attr = getattr(module, name, None)  # line 106
        yield (Left(InvalidLocator(f'{path} has no attribute {name}')) if attr is None else Right(attr))  # line 107

    @staticmethod  # line 113
    @_coconut_tco  # line 113
    def catch(f: '_coconut.typing.Callable[[], B]', exc: 'Type[E]') -> 'Either[E, B]':  # line 114
        try:  # line 115
            return Right(f())  # line 116
        except exc as e:  # line 117
            return _coconut_tail_call(Left, e)  # line 118

    @staticmethod  # line 120
    @do  # line 120
    def exports(modpath: 'str') -> 'Either[ImportFailure, amino.List[Any]]':  # line 122
        from amino.list import Lists  # line 123
        exports = yield Either.import_name(modpath, '__all__')  # line 124
        yield Lists.wrap(exports).traverse(lambda a: Either.import_name(modpath, a), Either)  # line 125

    @property  # line 127
    @_coconut_tco  # line 127
    def is_right(self) -> 'amino.Boolean':  # line 128
        return _coconut_tail_call(boolean.Boolean, isinstance(self, Right))  # line 129

    @property  # line 131
    @_coconut_tco  # line 131
    def is_left(self) -> 'amino.Boolean':  # line 132
        return _coconut_tail_call(boolean.Boolean, isinstance(self, Left))  # line 133

    @property  # line 135
    @_coconut_tco  # line 135
    def __left_value(self) -> 'A':  # line 136
        return _coconut_tail_call(cast, A, self.value)  # line 137

    @property  # line 139
    @_coconut_tco  # line 139
    def __right_value(self) -> 'B':  # line 140
        return _coconut_tail_call(cast, B, self.value)  # line 141

    def leffect(self, f: '_coconut.typing.Callable[[A], Any]') -> 'Either[A, B]':  # line 143
        if self.is_left:  # line 144
            f(self.__left_value)  # line 145
        return self  # line 146

    def bieffect(self, l: '_coconut.typing.Callable[[A], Any]', r: '_coconut.typing.Callable[[B], Any]') -> 'Either[A, B]':  # line 148
        self.cata(l, r)  # line 149
        return self  # line 150

    def cata(self, fl: '_coconut.typing.Callable[[A], C]', fr: '_coconut.typing.Callable[[B], C]') -> 'C':  # line 152
        return fl(self.__left_value) if self.is_left else fr(self.__right_value)  # line 153

    def bimap(self, fl: '_coconut.typing.Callable[[A], C]', fr: '_coconut.typing.Callable[[B], D]') -> 'Either[C, D]':  # line 155
        return Left(fl(self.__left_value)) if self.is_left else Right(fr(self.__right_value))  # line 156

    @_coconut_tco  # line 158
    def recover_with(self, f: '_coconut.typing.Callable[[A], Either[C, B]]') -> 'Either[C, B]':  # line 158
        return _coconut_tail_call(self.cata, f, Right)  # line 159

    @_coconut_tco  # line 161
    def right_or_map(self, f: '_coconut.typing.Callable[[A], C]') -> 'C':  # line 161
        return _coconut_tail_call(self.cata, f, I)  # line 162

    @_coconut_tco  # line 164
    def value_or(self, f: '_coconut.typing.Callable[[A], B]') -> 'B':  # line 164
        return _coconut_tail_call(self.cata, f, I)  # line 165

    @_coconut_tco  # line 167
    def left_or(self, f: '_coconut.typing.Callable[[B], A]') -> 'A':  # line 167
        return _coconut_tail_call(self.cata, I, f)  # line 168

    @_coconut_tco  # line 170
    def left_or_map(self, f: '_coconut.typing.Callable[[B], A]') -> 'A':  # line 170
        return _coconut_tail_call(self.cata, I, f)  # line 171

    @property  # line 173
    @_coconut_tco  # line 173
    def ljoin(self) -> 'Either[A, C]':  # line 174
        return _coconut_tail_call(self.right_or_map, Left)  # line 175

    @_coconut_tco  # line 177
    def __str__(self) -> 'str':  # line 177
        return _coconut_tail_call('{}({})'.format, self.__class__.__name__, self.value)  # line 178

    @_coconut_tco  # line 180
    def __repr__(self) -> 'str':  # line 180
        return _coconut_tail_call('{}({!r})'.format, self.__class__.__name__, self.value)  # line 181

    @property  # line 183
    def to_list(self) -> 'amino.List[B]':  # line 184
        return self.to_maybe.to_list  # line 185

    def lmap(self, f: '_coconut.typing.Callable[[A], C]') -> 'Either[C, B]':  # line 187
        return cast(Either, Left(f(self.__left_value))) if self.is_left else cast(Either, Right(self.__right_value))  # line 188

    @_coconut_tco  # line 190
    def get_or_raise(self) -> 'B':  # line 190
        def fail(err: 'A') -> 'B':  # line 191
            raise err if isinstance(err, Exception) else Exception(err)  # line 192
        return _coconut_tail_call(self.cata, fail, I)  # line 193

    @property  # line 195
    def fatal(self) -> 'B':  # line 196
        return self.get_or_raise  # line 197

    @_coconut_tco  # line 199
    def __iter__(self) -> 'Iterator[B]':  # line 199
        return _coconut_tail_call(iter, self.to_list)  # line 200

    @property  # line 202
    @_coconut_tco  # line 202
    def swap(self) -> 'Either[B, A]':  # line 203
        return _coconut_tail_call(self.cata, Right, Left)  # line 204

    @property  # line 206
    def json_repr(self) -> 'B':  # line 207
        return self.to_maybe.json_repr  # line 208

    @_coconut_tco  # line 210
    def accum_error(self, b: 'Either[A, C]') -> 'Either[A, C]':  # line 210
        return _coconut_tail_call(self.accum_error_f, lambda: b)  # line 211

    def accum_error_f(self, f: '_coconut.typing.Callable[[], Either[A, C]]') -> 'Either[A, C]':  # line 213
        @_coconut_tco  # line 214
        def acc(v: 'A') -> 'None':  # line 214
            return _coconut_tail_call(Monoid.fatal(type(v)).combine, self.__left_value, v)  # line 215
        return f().lmap(acc) if self.is_left else self  # line 216

    def filter_with(self, f: '_coconut.typing.Callable[[B], bool]', g: '_coconut.typing.Callable[[B], C]') -> 'Either[C, B]':  # line 218
        return self // (lambda a: Right(a) if f(a) else Left(g(a)))  # line 219

    @_coconut_tco  # line 221
    def left_contains(self, a: 'A') -> 'boolean.Boolean':  # line 221
        return _coconut_tail_call(boolean.Boolean, self.is_left and self.__left_value == a)  # line 222


class Right(_coconut.collections.namedtuple("Right", "value"), Generic[A, B], Either[A, B]):  # line 225
    __slots__ = ()  # line 225
    __ne__ = _coconut.object.__ne__  # line 225
    def __eq__(self, other: 'Any') -> 'bool':  # line 227
        return isinstance(other, Right) and self._Either__right_value == other._Either__right_value  # line 228


class Left(_coconut.collections.namedtuple("Left", "value"), Generic[A, B], Either[A, B]):  # line 231
    __slots__ = ()  # line 231
    __ne__ = _coconut.object.__ne__  # line 231
    def __eq__(self, other: 'Any') -> 'bool':  # line 233
        return isinstance(other, Left) and self._Either__left_value == other._Either__left_value  # line 234

__all__ = ('Either', 'Left', 'Right', 'ImportFailure', 'ImportException', 'InvalidLocator')  # line 236
